/*
 * Copyright (C) 2017 ~ 2017 Deepin Technology Co., Ltd.
 *
 * Author:     Iceyer <me@iceyer.net>
 *
 * Maintainer: Iceyer <me@iceyer.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "licenseinfo.h"

#include <QDateTime>

#include "licensedbus.h"
#include "licensekernel.h"

LicenseInfo::LicenseInfo(const LicenseKernel *kernel, QObject *parent): QObject(parent)
{
    Q_ASSERT(kernel != nullptr);

    m_kernel = kernel;
}

quint32 LicenseInfo::notifyLicenseState()
{
    auto state = licenseSatus();
    emit LicenseStateChange(state);
    return state;
}

quint32 LicenseInfo::licenseSatus()
{
    return m_kernel->isActivated() ? Authorized : Unauthorized;
}

QString LicenseInfo::validityBegin()
{
    return QDateTime::fromTime_t(m_kernel->validityBegin()).toString(Qt::ISODate);
}

QString LicenseInfo::validityEnd()
{
    return QDateTime::fromTime_t(m_kernel->validityEnd()).toString(Qt::ISODate);
}

QString LicenseInfo::organization()
{
    return m_kernel->organization();
}

QString LicenseInfo::product()
{
    if (LicenseKernel::FileLicense == m_kernel->type()) {
        return m_kernel->product();
    }
    return "";
}

QString LicenseInfo::copyright()
{
    return m_kernel->copyright();
}

QString LicenseInfo::edition()
{
    return m_kernel->edition();
}

QString LicenseInfo::machineCode()
{
    return m_kernel->machineCode();
}
