/*
 * Copyright (C) 2016 ~ 2017 Deepin Technology Co., Ltd.
 *
 * Author:     Iceyer <me@iceyer.net>
 *
 * Maintainer: Iceyer <me@iceyer.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QObject>

class LicenseKernelPrivate;
class LicenseKernel: QObject
{
    Q_OBJECT
public:
    enum LicenseType {
        InvalidLicense = 0x00000000,
        FileLicense = 0x80000000, // default no authorize type
        KeyLicense  = 0x80000080,
    };

    explicit LicenseKernel(QObject *parent = Q_NULLPTR);
    ~LicenseKernel();

    void loadFile(char *keyData);
    void loadKey(const char *serialkey);
    void deactive();
    bool isActivated() const;

    LicenseType type() const;
    QString organization() const;
    QString product() const;
    QString copyright() const;
    QString edition() const;
    QString authorizeCode() const;
    QString signatureRSA() const;
    QString machineCode() const;
    quint64 validityBegin() const;
    quint64 validityEnd() const;
    QByteArray licenseData() const;

    static qint64 licenseSize();

private:
    QScopedPointer<LicenseKernelPrivate> d_ptr;
    Q_DECLARE_PRIVATE_D(qGetPtrHelper(d_ptr), LicenseKernel)
};

